@@ -7,7 +7,7 @@ use sodiumoxide::crypto::kx::{x25519blake2b, PublicKey, SecretKey};
 use sodiumoxide::randombytes;
 
 use super::SEGMENT_SIZE;
-use crate::error::ApiError;
+use crate::error::Crypt4GHError;
 use crate::keys::get_public_key_from_private_key;
 use crate::Keys;
 const MAGIC_NUMBER: &[u8; 8] = b"crypt4gh";
@@ -66,7 +66,11 @@ pub fn make_packet_data_edit_list(edit_list: Vec<usize>) -> Vec<u8> {
 	.concat()
 }
 
-fn encrypt_x25519_chacha20_poly1305(data: &[u8], seckey: &[u8], recipient_pubkey: &[u8]) -> Result<Vec<u8>, ApiError> {
+fn encrypt_x25519_chacha20_poly1305(
+	data: &[u8],
+	seckey: &[u8],
+	recipient_pubkey: &[u8],
+) -> Result<Vec<u8>, Crypt4GHError> {
 	crate::init();
 	let pubkey = get_public_key_from_private_key(seckey)?;
 
@@ -85,17 +89,17 @@ fn encrypt_x25519_chacha20_poly1305(data: &[u8], seckey: &[u8], recipient_pubkey
 	);
 
 	// X25519 shared key
-	let server_pk = PublicKey::from_slice(pubkey.as_ref()).ok_or(ApiError::BadServerPublicKey)?;
-	let server_sk = SecretKey::from_slice(&seckey[0..32]).ok_or(ApiError::BadServerPrivateKey)?;
-	let client_pk = PublicKey::from_slice(recipient_pubkey).ok_or(ApiError::BadClientPublicKey)?;
-	let (_, shared_key) =
-		x25519blake2b::server_session_keys(&server_pk, &server_sk, &client_pk).map_err(|_| ApiError::BadSharedKey)?;
+	let server_pk = PublicKey::from_slice(pubkey.as_ref()).ok_or(Crypt4GHError::BadServerPublicKey)?;
+	let server_sk = SecretKey::from_slice(&seckey[0..32]).ok_or(Crypt4GHError::BadServerPrivateKey)?;
+	let client_pk = PublicKey::from_slice(recipient_pubkey).ok_or(Crypt4GHError::BadClientPublicKey)?;
+	let (_, shared_key) = x25519blake2b::server_session_keys(&server_pk, &server_sk, &client_pk)
+		.map_err(|_| Crypt4GHError::BadSharedKey)?;
 	log::debug!("   shared key: {:02x?}", shared_key.0.iter().format(""));
 
 	// Nonce & chacha20 key
 	let nonce =
-		chacha20poly1305_ietf::Nonce::from_slice(&randombytes::randombytes(12)).ok_or(ApiError::NoRandomNonce)?;
-	let key = chacha20poly1305_ietf::Key::from_slice(shared_key.as_ref()).ok_or(ApiError::BadKey)?;
+		chacha20poly1305_ietf::Nonce::from_slice(&randombytes::randombytes(12)).ok_or(Crypt4GHError::NoRandomNonce)?;
+	let key = chacha20poly1305_ietf::Key::from_slice(shared_key.as_ref()).ok_or(Crypt4GHError::BadKey)?;
 
 	Ok(vec![
 		pubkey,
@@ -112,7 +116,7 @@ fn encrypt_x25519_chacha20_poly1305(data: &[u8], seckey: &[u8], recipient_pubkey
 ///
 /// * `packet` is a vector of bytes of information to be encrypted
 /// * `keys` is a unique collection of keys with `key.method` == 0
-pub fn encrypt(packet: &[u8], keys: &HashSet<Keys>) -> Result<Vec<Vec<u8>>, ApiError> {
+pub fn encrypt(packet: &[u8], keys: &HashSet<Keys>) -> Result<Vec<Vec<u8>>, Crypt4GHError> {
 	keys.iter()
 		.filter(|key| key.method == 0)
 		.map(
@@ -154,7 +158,7 @@ fn decrypt(
 			Ok(decrypted_packet) => decrypted_packets.push(decrypted_packet),
 			Err(e) => {
 				log::warn!("Ignoring packet because: {}", e);
-				ignored_packets.push(packet)
+				ignored_packets.push(packet);
 			},
 		}
 	}
@@ -162,9 +166,9 @@ fn decrypt(
 	(decrypted_packets, ignored_packets)
 }
 
-fn decrypt_packet(packet: &[u8], keys: &[Keys], sender_pubkey: &Option<Vec<u8>>) -> Result<Vec<u8>, ApiError> {
+fn decrypt_packet(packet: &[u8], keys: &[Keys], sender_pubkey: &Option<Vec<u8>>) -> Result<Vec<u8>, Crypt4GHError> {
 	let packet_encryption_method =
-		bincode::deserialize::<u32>(packet).map_err(|_| ApiError::ReadPacketEncryptionMethod)?;
+		bincode::deserialize::<u32>(packet).map_err(|_| Crypt4GHError::ReadPacketEncryptionMethod)?;
 	log::debug!("Header Packet Encryption Method: {}", packet_encryption_method);
 
 	for key in keys {
@@ -175,28 +179,28 @@ fn decrypt_packet(packet: &[u8], keys: &[Keys], sender_pubkey: &Option<Vec<u8>>)
 		match packet_encryption_method {
 			0 => return decrypt_x25519_chacha20_poly1305(&packet[4..], &key.privkey, sender_pubkey),
 			1 => unimplemented!("AES-256-GCM support is not implemented"),
-			n => return Err(ApiError::BadHeaderEncryptionMethod(n)),
+			n => return Err(Crypt4GHError::BadHeaderEncryptionMethod(n)),
 		}
 	}
 
-	Err(ApiError::UnableToEncryptPacket)
+	Err(Crypt4GHError::UnableToEncryptPacket)
 }
 
 fn decrypt_x25519_chacha20_poly1305(
 	encrypted_part: &[u8],
 	privkey: &[u8],
 	sender_pubkey: &Option<Vec<u8>>,
-) -> Result<Vec<u8>, ApiError> {
+) -> Result<Vec<u8>, Crypt4GHError> {
 	log::debug!("    my secret key: {:02x?}", &privkey[0..32].iter().format(""));
 
 	let peer_pubkey = &encrypted_part[0..32];
 
 	if sender_pubkey.is_some() && sender_pubkey.clone().unwrap().as_slice() != peer_pubkey {
-		return Err(ApiError::InvalidPeerPubPkey);
+		return Err(Crypt4GHError::InvalidPeerPubPkey);
 	}
 
 	let nonce = sodiumoxide::crypto::aead::chacha20poly1305_ietf::Nonce::from_slice(&encrypted_part[32..44])
-		.ok_or(ApiError::NoNonce)?;
+		.ok_or(Crypt4GHError::NoNonce)?;
 	let packet_data = &encrypted_part[44..];
 
 	log::debug!("    peer pubkey: {:02x?}", peer_pubkey.iter().format(""));
@@ -209,25 +213,25 @@ fn decrypt_x25519_chacha20_poly1305(
 
 	// X25519 shared key
 	let pubkey = get_public_key_from_private_key(privkey)?;
-	let client_pk = PublicKey::from_slice(&pubkey).ok_or(ApiError::BadClientPublicKey)?;
-	let client_sk = SecretKey::from_slice(&privkey[0..32]).ok_or(ApiError::BadClientPrivateKey)?;
-	let server_pk = PublicKey::from_slice(peer_pubkey).ok_or(ApiError::BadServerPublicKey)?;
-	let (shared_key, _) =
-		x25519blake2b::client_session_keys(&client_pk, &client_sk, &server_pk).map_err(|_| ApiError::BadSharedKey)?;
+	let client_pk = PublicKey::from_slice(&pubkey).ok_or(Crypt4GHError::BadClientPublicKey)?;
+	let client_sk = SecretKey::from_slice(&privkey[0..32]).ok_or(Crypt4GHError::BadClientPrivateKey)?;
+	let server_pk = PublicKey::from_slice(peer_pubkey).ok_or(Crypt4GHError::BadServerPublicKey)?;
+	let (shared_key, _) = x25519blake2b::client_session_keys(&client_pk, &client_sk, &server_pk)
+		.map_err(|_| Crypt4GHError::BadSharedKey)?;
 	log::debug!("shared key: {:02x?}", shared_key.0.iter().format(""));
 
 	// Chacha20_Poly1305
-	let key = chacha20poly1305_ietf::Key::from_slice(&shared_key.0).ok_or(ApiError::BadSharedKey)?;
-	chacha20poly1305_ietf::open(packet_data, None, &nonce, &key).map_err(|_| ApiError::InvalidData)
+	let key = chacha20poly1305_ietf::Key::from_slice(&shared_key.0).ok_or(Crypt4GHError::BadSharedKey)?;
+	chacha20poly1305_ietf::open(packet_data, None, &nonce, &key).map_err(|_| Crypt4GHError::InvalidData)
 }
 
-fn partition_packets(packets: Vec<Vec<u8>>) -> Result<HeaderPackets, ApiError> {
+fn partition_packets(packets: Vec<Vec<u8>>) -> Result<HeaderPackets, Crypt4GHError> {
 	let mut enc_packets = Vec::new();
 	let mut edits = None;
 
 	for packet in packets {
 		let packet_type =
-			bincode::deserialize::<HeaderPacketType>(&packet[0..4]).map_err(|_| ApiError::InvalidPacketType)?;
+			bincode::deserialize::<HeaderPacketType>(&packet[0..4]).map_err(|_| Crypt4GHError::InvalidPacketType)?;
 
 		match packet_type {
 			HeaderPacketType::DataEnc => {
@@ -236,7 +240,7 @@ fn partition_packets(packets: Vec<Vec<u8>>) -> Result<HeaderPackets, ApiError> {
 			HeaderPacketType::EditList => {
 				match edits {
 					None => edits = Some(packet[4..].to_vec()),
-					Some(_) => return Err(ApiError::TooManyEditListPackets),
+					Some(_) => return Err(Crypt4GHError::TooManyEditListPackets),
 				};
 			},
 		}
@@ -248,28 +252,28 @@ fn partition_packets(packets: Vec<Vec<u8>>) -> Result<HeaderPackets, ApiError> {
 	})
 }
 
-fn parse_enc_packet(packet: &[u8]) -> Result<Vec<u8>, ApiError> {
+fn parse_enc_packet(packet: &[u8]) -> Result<Vec<u8>, Crypt4GHError> {
 	match packet[0..4] {
 		[0, 0, 0, 0] => Ok(packet[4..].to_vec()),
-		_ => Err(ApiError::UnsupportedEncryptionMethod(
+		_ => Err(Crypt4GHError::UnsupportedEncryptionMethod(
 			bincode::deserialize::<u32>(&packet[0..4]).expect("Unable to deserialize bulk encryption method"),
 		)),
 	}
 }
 
-fn parse_edit_list_packet(packet: &[u8]) -> Result<Vec<u64>, ApiError> {
-	let nb_lengths: u32 = bincode::deserialize::<u32>(packet).map_err(|_| ApiError::NoEditListLength)?;
+fn parse_edit_list_packet(packet: &[u8]) -> Result<Vec<u64>, Crypt4GHError> {
+	let nb_lengths: u32 = bincode::deserialize::<u32>(packet).map_err(|_| Crypt4GHError::NoEditListLength)?;
 
 	log::info!("Edit list length: {}", nb_lengths);
 	log::info!("packet content length: {}", packet.len() - 4);
 
 	if ((packet.len() as u32) - 4) < (8 * nb_lengths) {
-		return Err(ApiError::InvalidEditList);
+		return Err(Crypt4GHError::InvalidEditList);
 	}
 
 	(4..nb_lengths * 8)
 		.step_by(8)
-		.map(|i| bincode::deserialize::<u64>(&packet[i as usize..]).map_err(|_| ApiError::InvalidEditList))
+		.map(|i| bincode::deserialize::<u64>(&packet[i as usize..]).map_err(|_| Crypt4GHError::InvalidEditList))
 		.collect()
 }
 
@@ -281,11 +285,11 @@ pub fn deconstruct_header_body(
 	encrypted_packets: Vec<Vec<u8>>,
 	keys: &[Keys],
 	sender_pubkey: &Option<Vec<u8>>,
-) -> Result<DecryptedHeaderPackets, ApiError> {
+) -> Result<DecryptedHeaderPackets, Crypt4GHError> {
 	let (packets, _) = decrypt(encrypted_packets, keys, sender_pubkey);
 
 	if packets.is_empty() {
-		return Err(ApiError::NoSupportedEncryptionMethod);
+		return Err(Crypt4GHError::NoSupportedEncryptionMethod);
 	}
 
 	let HeaderPackets {
@@ -296,7 +300,7 @@ pub fn deconstruct_header_body(
 	let session_keys = data_enc_packets
 		.into_iter()
 		.map(|d| parse_enc_packet(&d))
-		.collect::<Result<Vec<_>, ApiError>>()?;
+		.collect::<Result<Vec<_>, Crypt4GHError>>()?;
 
 	let edit_list = match edit_list_packet {
 		Some(packet) => Some(parse_edit_list_packet(&packet)?),
@@ -314,8 +318,9 @@ pub fn deconstruct_header_body(
 /// Reads the magic number, the version and the number of packets from the bytes.
 pub fn deconstruct_header_info(
 	header_info_file: &[u8; std::mem::size_of::<HeaderInfo>()],
-) -> Result<HeaderInfo, ApiError> {
-	let header_info = bincode::deserialize::<HeaderInfo>(header_info_file).map_err(|e| ApiError::ReadHeaderError(e))?;
+) -> Result<HeaderInfo, Crypt4GHError> {
+	let header_info =
+		bincode::deserialize::<HeaderInfo>(header_info_file).map_err(|e| Crypt4GHError::ReadHeaderError(e))?;
 
 	assert!(
 		&header_info.magic_number == MAGIC_NUMBER,
@@ -339,13 +344,13 @@ pub fn reencrypt(
 	keys: &[Keys],
 	recipient_keys: &HashSet<Keys>,
 	trim: bool,
-) -> Result<Vec<Vec<u8>>, ApiError> {
+) -> Result<Vec<Vec<u8>>, Crypt4GHError> {
 	log::info!("Reencrypting the header");
 
 	let (decrypted_packets, mut ignored_packets) = decrypt(header_packets, keys, &None);
 
 	if decrypted_packets.is_empty() {
-		Err(ApiError::NoValidHeaderPacket)
+		Err(Crypt4GHError::NoValidHeaderPacket)
 	}
 	else {
 		let mut packets: Vec<Vec<u8>> = decrypted_packets
@@ -371,27 +376,29 @@ pub fn rearrange<'a>(
 	range_start: usize,
 	range_span: Option<usize>,
 	sender_pubkey: &Option<Vec<u8>>,
-) -> Result<(Vec<Vec<u8>>, impl Iterator<Item = bool> + 'a), ApiError> {
+) -> Result<(Vec<Vec<u8>>, impl Iterator<Item = bool> + 'a), Crypt4GHError> {
 	log::info!("Rearranging the header");
 
 	log::debug!("    Start coordinate: {}", range_start);
-	if let Some(span) = range_span {
-		log::debug!("    End coordinate: {}", range_start + span);
-		assert!(span > 0, "Span should be greater than 0")
-	}
-	else {
-		log::debug!("    End coordinate: EOF")
-	}
+	range_span.map_or_else(
+		|| {
+			log::debug!("    End coordinate: EOF");
+		},
+		|span| {
+			log::debug!("    End coordinate: {}", range_start + span);
+			assert!(span > 0, "Span should be greater than 0");
+		},
+	);
 	log::debug!("    Segment size: {}", SEGMENT_SIZE);
 
 	if range_start == 0 && range_span.is_none() {
-		return Err(ApiError::Done);
+		return Err(Crypt4GHError::Done);
 	}
 
 	let (decrypted_packets, _) = decrypt(header_packets, &keys, sender_pubkey);
 
 	if decrypted_packets.is_empty() {
-		return Err(ApiError::NoValidHeaderPacket);
+		return Err(Crypt4GHError::NoValidHeaderPacket);
 	}
 
 	let HeaderPackets {
@@ -447,7 +454,7 @@ pub fn rearrange<'a>(
 	let final_packets = packets
 		.into_iter()
 		.map(|packet| encrypt(&packet, &hash_keys).map(|encrypted_packets| encrypted_packets.concat()))
-		.collect::<Result<Vec<Vec<u8>>, ApiError>>()?;
+		.collect::<Result<Vec<Vec<u8>>, Crypt4GHError>>()?;
 
 	Ok((final_packets, segment_oracle))
 }
